use strict;
use warnings;
use init_params;

sub write_params {

    # Purpose: write out parameter file
    #
    # Input arguments:
    #   $param_file - input file
    #   $auto_ref   - reference to hash of control parameters
    #   $po2r_ref   - reference to hash of PO2R input parameters
    #   $p3dr_ref   - reference to hash of P3DR input parameters
    #   $pcut_ref   - reference to hash of PCUT input parameters
    #   $psf_ref    - reference to hash of PSF input parameters
    #   $ppft_ref   - reference to hash of PSF input parameters
    #   $data_ref   - reference to array of data files
    #
    # Return values:
    #   1 if file successfully written, 0 otherwise

    my ($param_file, $auto_ref, $po2r_ref, $p3dr_ref,
	$pcut_ref, $psf_ref, $ppft_ref, $data_ref) = @_;

    unless (open(FH, ">$param_file")) {
	print "Unable to write file $param_file";
	return(0);
    }

    foreach my $key (sort keys %{$auto_ref}) {
	if(valid_key('auto', $key)) {
	    print FH "auto\t$key\t$$auto_ref{$key}\n";
	}
    }
    print FH "\n";
    foreach my $key (sort keys %{$po2r_ref}) {
	if(valid_key('po2r', $key)) {
	    print FH "po2r\t$key\t$$po2r_ref{$key}\n";
	}
    }
    print FH "\n";
    foreach my $key (sort keys %{$p3dr_ref}) {
	if(valid_key('p3dr', $key)) {
	    print FH "p3dr\t$key\t$$p3dr_ref{$key}\n";
	}
    }
    print FH "\n";
    foreach my $key (sort keys %{$pcut_ref}) {
	if(valid_key('pcut', $key)) {
	    print FH "pcut\t$key\t$$pcut_ref{$key}\n";
	}
    }
    print FH "\n";
    foreach my $key (sort keys %{$psf_ref}) {
	if(valid_key('psf', $key)) {
	    print FH "psf\t$key\t$$psf_ref{$key}\n";
	}
    }
    print FH "\n";
    foreach my $key (sort keys %{$ppft_ref}) {
	if(valid_key('ppft', $key)) {
	    print FH "ppft\t$key\t$$ppft_ref{$key}\n";
	}
    }
    print FH "\n";

    foreach my $file (sort @{$data_ref}) {
	print FH "data\t$file\n";
    }

    close(FH);
    return(1);
}
1;

	




