      subroutine maptempfac(fft_klh, hkl_max, tempfac, pixsize,
     $     units, lb, ub)

C     Apply temperature factor to FT of map fft_klh. Since the
C     temperature factor is a function purely of the magnitude of the
C     wave vector, rather than the direction, it only needs to be
C     explicitly calculated for wave vectors in which all of the
C     components are non-negative. It is assumed that the bounds of the
C     3rd dimension, lb and ub, are both non-negative.
      
      implicit none
      
      integer, intent(in)    :: hkl_max, units, lb, ub
      real, intent(in)       :: tempfac, pixsize
      complex, intent(inout) :: fft_klh(-hkl_max:hkl_max,
     $     -hkl_max:hkl_max, lb:ub)
      
      integer ix, iy, iz
      real    d_star2, sumsq, t, tmp, tmp2
      
      if(units.eq.2) then
         tmp =  1.0/((hkl_max*2+1)*pixsize) ! already in nm units
      else
         tmp = 10.0/((hkl_max*2+1)*pixsize) ! convert to nm units
      endif
      
      tmp2 = tmp*tmp

      do iz = lb, ub
         do iy = 0, hkl_max
            do ix = 0, hkl_max

               sumsq = real(ix*ix + iy*iy + iz*iz)
               d_star2 = sumsq * tmp2
               t = exp ((tempfac/400.0) * d_star2)
               
               fft_klh(ix,iy,iz) = fft_klh(ix,iy,iz) * t

               if (ix .ne. 0 .and. iy .ne.0) then
                  fft_klh(-ix,iy,iz) = fft_klh(-ix,iy,iz)   * t
                  fft_klh(ix,-iy,iz) = fft_klh(ix,-iy,iz)   * t
                  fft_klh(-ix,-iy,iz) = fft_klh(-ix,-iy,iz) * t
               elseif (ix .eq. 0 .and. iy .ne.0) then
                  fft_klh(ix,-iy,iz) = fft_klh(ix,-iy,iz) * t
               elseif (ix .ne. 0 .and. iy .eq.0) then
                  fft_klh(-ix,iy,iz) = fft_klh(-ix,iy,iz) * t
               endif

            enddo
         enddo
      enddo
      
      return
      end subroutine maptempfac
